"use strict";
const EXPORTED_SYMBOLS = ["AddonManager"];
const {
    classes: Cc,
    interfaces: Ci,
    results: Cr,
    utils: Cu
} = Components;
Cu.import("resource://gre/modules/Services.jsm");
const AddonManager = {
    SCOPE_PROFILE: 1,
    SCOPE_APPLICATION: 4,
    _started: false,
    _applyCallback: function AM__applyCallback(aCallback) {
        if (!aCallback) {
            return;
        }
        try {
            aCallback.apply(null, Array.slice(arguments, 1));
        } catch (e) {
            Cu.reportError(e);
        }
    },
    get gre_AddonManager() {
        delete this.gre_AddonManager;
        return this.gre_AddonManager = Cu.import("resource://gre/modules/AddonManager.jsm", {}).AddonManager;
    },
    _getInstallRdfContent: function AM__getInstallRdfContent(aExtensionURI) {
        if (aExtensionURI instanceof Ci.nsIFile) {
            let installRDFFile = aExtensionURI;
            installRDFFile.append("install.rdf");
            let content = "";
            try {
                let inputStream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(Ci.nsIFileInputStream);
                inputStream.init(installRDFFile, 1, 0, inputStream.CLOSE_ON_EOF);
                let fileSize = inputStream.available();
                let cvstream = Cc["@mozilla.org/intl/converter-input-stream;1"].createInstance(Ci.nsIConverterInputStream);
                cvstream.init(inputStream, "UTF-8", fileSize, Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
                let data = {};
                cvstream.readString(fileSize, data);
                content = data.value;
                cvstream.close();
            } catch (e) {
                Cu.reportError(e);
            }
            return content;
        }
        let installFileURI = Services.io.newURI(aExtensionURI.resolve("install.rdf"), null, null);
        let installFileChannel = Services.io.newChannelFromURI(installFileURI);
        let installFileStream = installFileChannel.open();
        let content = "";
        let streamSize = installFileStream.available();
        let converterStream = Cc["@mozilla.org/intl/converter-input-stream;1"].createInstance(Ci.nsIConverterInputStream);
        try {
            let data = {};
            converterStream.init(installFileStream, "UTF-8", streamSize, Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
            converterStream.readString(streamSize, data);
            content = data.value;
        } catch (e) {
            Cu.reportError(e);
        } finally {
            converterStream.close();
        }
        return content;
    },
    getAddonVersion: function AM_getAddonVersion(aExtensionURI) {
        let installRdfContent = this._getInstallRdfContent(aExtensionURI);
        if (installRdfContent) {
            let version = installRdfContent.match(/<em:version>([^<]*)<\/em:version>/);
            if (version && /^\d+\.\d+/.test(version[1])) {
                return version[1];
            }
        }
        throw new Error("AddonManager: can't get addon version from install.rdf");
    },
    getAddonId: function AM_getAddonId(aExtensionURI) {
        let installRdfContent = this._getInstallRdfContent(aExtensionURI);
        if (installRdfContent) {
            let addonId = installRdfContent.match(/<em:id>([^<]*)<\/em:id>/);
            if (addonId && addonId[1]) {
                return addonId[1];
            }
        }
        throw new Error("AddonManager: can't get addon id from install.rdf");
    },
    disableAddonByID: function AM_disableAddonByID(aAddonId, aCallback) {
        this.gre_AddonManager.getAddonByID(aAddonId, function AM_disableAddonByID_callback(aAddon) {
            if (aAddon) {
                aAddon.userDisabled = true;
            }
            this._applyCallback(aCallback);
        }.bind(this));
    },
    uninstallAddonsByIDs: function AM_uninstallAddonsByIDs(aAddonIds, aRestartBrowser, aCallback) {
        let onUninstall = function onUninstall(addons, restart) {
            this._applyCallback(aCallback);
            if (Boolean(aRestartBrowser) && (restart || addons && addons.length)) {
                let timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
                timer.initWithCallback({
                    notify: function AM_uninstallAddonsByIDs_onUninstall() {
                        const nsIAppStartup = Ci.nsIAppStartup;
                        Cc["@mozilla.org/toolkit/app-startup;1"].getService(nsIAppStartup).quit(nsIAppStartup.eForceQuit | nsIAppStartup.eRestart);
                    }
                }, 150, timer.TYPE_ONE_SHOT);
            }
        }.bind(this);
        let addonsUninstallCallback = false;
        try {
            AddonManager.getAddonsByIDs(aAddonIds, function (addons) {
                if (addonsUninstallCallback) {
                    return;
                }
                addonsUninstallCallback = true;
                addons.forEach(function (addon) {
                    try {
                        addon.uninstall();
                    } catch (e) {
                    }
                });
                onUninstall(addons);
            });
        } catch (e) {
        }
        let timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
        timer.initWithCallback({
            notify: function AM_uninstallAddonsByIDs_wait() {
                if (!addonsUninstallCallback) {
                    addonsUninstallCallback = true;
                    onUninstall(null, true);
                }
            }
        }, 3000, timer.TYPE_ONE_SHOT);
    },
    getAddonsByIDs: function AM_getAddonsByIDs(aIds, aCallback) {
        this.gre_AddonManager.getAddonsByIDs(aIds, aCallback);
    },
    _addonListeners: [],
    startup: function AM_startup() {
        if (this._started) {
            return;
        }
        this._started = true;
        let me = this;
        [
            "onEnabling",
            "onEnabled",
            "onDisabling",
            "onDisabled",
            "onInstalling",
            "onInstalled",
            "onUninstalling",
            "onUninstalled",
            "onOperationCancelled",
            "onUpdateAvailable",
            "onNoUpdateAvailable",
            "onUpdateFinished",
            "onCompatibilityUpdateAvailable",
            "onNoCompatibilityUpdateAvailable",
            "onPropertyChanged"
        ].forEach(function (aEvent) {
            me[aEvent] = function () {
                me._notifyAddonListeners.apply(me, [aEvent].concat(Array.slice(arguments)));
            };
        });
        this.gre_AddonManager.addAddonListener(this);
    },
    shutdown: function AM_shutdown() {
        if (!this._started) {
            return;
        }
        this._started = false;
        this.gre_AddonManager.removeAddonListener(this);
        this._addonListeners = null;
        this._watchingAddons = null;
    },
    observe: function AM_observe(aSubject, aTopic, aData) {
        switch (aTopic) {
        case "browser-ui-startup-complete":
            Services.obs.removeObserver(this, "browser-ui-startup-complete");
            Services.obs.addObserver(this, "xpcom-shutdown", false);
            this.startup();
            break;
        case "xpcom-shutdown":
            Services.obs.removeObserver(this, "xpcom-shutdown");
            this.shutdown();
            break;
        default:
            break;
        }
    },
    onAddonEvent: function AM_onAddonEvent(aEventType, aAddon, aPendingRestart) {
        let addonId = aAddon.id;
        let watchingAddon = this._watchingAddons[addonId] || null;
        if (!watchingAddon) {
            return;
        }
        watchingAddon.installed = aEventType !== "onUninstalling";
    },
    _watchingAddons: Object.create(null),
    watchAddonUninstall: function AM_watchAddonUninstall(aAddonId) {
        this._watchingAddons[aAddonId] = {
            __proto__: null,
            installed: null
        };
        this.addAddonListener(this);
    },
    isAddonUserDisabled: function AM_isAddonUserDisabled(aAddonId, aSleepTimeout) {
        let result = null;
        let callback = false;
        this.gre_AddonManager.getAddonByID(aAddonId, function AM_isAddonUserDisabled_callback(aAddon) {
            callback = true;
            if (aAddon) {
                result = Boolean(aAddon.userDisabled);
            }
        });
        sleep(aSleepTimeout ? aSleepTimeout : 10000, function () {
            return !callback;
        });
        return result;
    },
    isAddonUninstalling: function AM_isAddonUninstalling(aAddonId) {
        let watchingAddon = this._watchingAddons[aAddonId] || null;
        return watchingAddon && watchingAddon.installed === false;
    },
    addAddonListener: function AM_addAddonListener(aListener) {
        if (!this._addonListeners.some(function (listener) {
                return listener == aListener;
            })) {
            this._addonListeners.push(aListener);
        }
    },
    removeAddonListener: function AM_removeAddonListener(aListener) {
        this._addonListeners = this._addonListeners.filter(function (listener) {
            return listener != aListener;
        });
    },
    _notifyAddonListeners: function AM__notifyAddonListeners() {
        let args = arguments;
        this._addonListeners.forEach(function (aListener) {
            try {
                aListener.onAddonEvent.apply(aListener, args);
            } catch (e) {
                Cu.reportError("AddonManager._notifyAddonListeners threw exception " + "when calling Listener.onAddonEvent: " + e);
            }
        });
    }
};
function sleep(aTimeout, aConditionFunction) {
    let func = typeof aConditionFunction == "function" ? aConditionFunction : function () {
        return true;
    };
    let timeout = 1;
    if (typeof aTimeout == "number" && aTimeout > 0) {
        timeout = aTimeout;
    }
    let t = Date.now();
    let conditionFunc = function conditionFunc() {
        return Date.now() - t < timeout && func();
    };
    let thread = Cc["@mozilla.org/thread-manager;1"].getService().currentThread;
    while (conditionFunc()) {
        thread.processNextEvent(true);
    }
}
Services.obs.addObserver(AddonManager, "browser-ui-startup-complete", false);
